@@ -4,7 +4,7 @@
 1D Wasserstein barycenter demo
 ==============================
 
-This example illustrates the computation of regularized Wassersyein Barycenter
+This example illustrates the computation of regularized Wasserstein Barycenter
 as proposed in [3].
 
 
@@ -80,6 +80,7 @@ plt.show()
 ##############################################################################
 # Barycentric interpolation
 # -------------------------
+
 #%% barycenter interpolation
 
 n_alpha = 11
@@ -99,7 +100,7 @@ for i in range(n_alpha):
 #%% plot interpolation
 plt.figure(2)
 
-cmap = plt.cm.get_cmap('viridis')
+cmap = plt.get_cmap('viridis')
 verts = []
 zs = alpha_list
 for i, z in enumerate(zs):
@@ -121,7 +122,7 @@ plt.title('Barycenter interpolation with l2')
 plt.tight_layout()
 
 plt.figure(3)
-cmap = plt.cm.get_cmap('viridis')
+cmap = plt.get_cmap('viridis')
 verts = []
 zs = alpha_list
 for i, z in enumerate(zs):
