@@ -17,11 +17,12 @@ import warnings
 
 from . import cvx
 from .cvx import barycenter
+from .dmmot import dmmot_monge_1dgrid_loss, dmmot_monge_1dgrid_optimize
 
 # import compiled emd
 from .emd_wrap import emd_c, check_result, emd_1d_sorted
-from .solver_1d import (emd_1d, emd2_1d, wasserstein_1d, 
-                        binary_search_circle, wasserstein_circle, 
+from .solver_1d import (emd_1d, emd2_1d, wasserstein_1d,
+                        binary_search_circle, wasserstein_circle,
                         semidiscrete_wasserstein2_unif_circle)
 
 from ..utils import dist, list_to_array
@@ -30,7 +31,8 @@ from ..backend import get_backend
 
 __all__ = ['emd', 'emd2', 'barycenter', 'free_support_barycenter', 'cvx', ' emd_1d_sorted',
            'emd_1d', 'emd2_1d', 'wasserstein_1d', 'generalized_free_support_barycenter',
-           'binary_search_circle', 'wasserstein_circle', 'semidiscrete_wasserstein2_unif_circle']
+           'binary_search_circle', 'wasserstein_circle', 'semidiscrete_wasserstein2_unif_circle',
+           'dmmot_monge_1dgrid_loss', 'dmmot_monge_1dgrid_optimize']
 
 
 def check_number_threads(numThreads):
@@ -200,7 +202,7 @@ def estimate_dual_null_weights(alpha0, beta0, a, b, M):
     return center_ot_dual(alpha, beta, a, b)
 
 
-def emd(a, b, M, numItermax=100000, log=False, center_dual=True, numThreads=1):
+def emd(a, b, M, numItermax=100000, log=False, center_dual=True, numThreads=1, check_marginals=True):
     r"""Solves the Earth Movers distance problem and returns the OT matrix
 
 
@@ -253,10 +255,14 @@ def emd(a, b, M, numItermax=100000, log=False, center_dual=True, numThreads=1):
         Otherwise returns only the optimal transportation matrix.
     center_dual: boolean, optional (default=True)
         If True, centers the dual potential using function
-        :ref:`center_ot_dual`.
+        :py:func:`ot.lp.center_ot_dual`.
     numThreads: int or "max", optional (default=1, i.e. OpenMP is not used)
         If compiled with OpenMP, chooses the number of threads to parallelize.
         "max" selects the highest number possible.
+    check_marginals: bool, optional (default=True)
+        If True, checks that the marginals mass are equal. If False, skips the
+        check.
+
 
     Returns
     -------
@@ -296,17 +302,24 @@ def emd(a, b, M, numItermax=100000, log=False, center_dual=True, numThreads=1):
     ot.optim.cg : General regularized OT
     """
 
-    # convert to numpy if list
     a, b, M = list_to_array(a, b, M)
+    nx = get_backend(M, a, b)
 
-    a0, b0, M0 = a, b, M
-    if len(a0) != 0:
-        type_as = a0
-    elif len(b0) != 0:
-        type_as = b0
+    if len(a) != 0:
+        type_as = a
+    elif len(b) != 0:
+        type_as = b
     else:
-        type_as = M0
-    nx = get_backend(M0, a0, b0)
+        type_as = M
+
+    # if empty array given then use uniform distributions
+    if len(a) == 0:
+        a = nx.ones((M.shape[0],), type_as=type_as) / M.shape[0]
+    if len(b) == 0:
+        b = nx.ones((M.shape[1],), type_as=type_as) / M.shape[1]
+
+    # store original tensors
+    a0, b0, M0 = a, b, M
 
     # convert to numpy
     M, a, b = nx.to_numpy(M, a, b)
@@ -326,8 +339,10 @@ def emd(a, b, M, numItermax=100000, log=False, center_dual=True, numThreads=1):
         "Dimension mismatch, check dimensions of M with a and b"
 
     # ensure that same mass
-    np.testing.assert_almost_equal(a.sum(0),
-                                   b.sum(0), err_msg='a and b vector must have the same sum')
+    if check_marginals:
+        np.testing.assert_almost_equal(a.sum(0),
+                                       b.sum(0), err_msg='a and b vector must have the same sum',
+                                       decimal=6)
     b = b * a.sum() / b.sum()
 
     asel = a != 0
@@ -365,7 +380,7 @@ def emd(a, b, M, numItermax=100000, log=False, center_dual=True, numThreads=1):
 
 def emd2(a, b, M, processes=1,
          numItermax=100000, log=False, return_matrix=False,
-         center_dual=True, numThreads=1):
+         center_dual=True, numThreads=1, check_marginals=True):
     r"""Solves the Earth Movers distance problem and returns the loss
 
     .. math::
@@ -418,10 +433,14 @@ def emd2(a, b, M, processes=1,
         If True, returns the optimal transportation matrix in the log.
     center_dual: boolean, optional (default=True)
         If True, centers the dual potential using function
-        :ref:`center_ot_dual`.
+        :py:func:`ot.lp.center_ot_dual`.
     numThreads: int or "max", optional (default=1, i.e. OpenMP is not used)
         If compiled with OpenMP, chooses the number of threads to parallelize.
         "max" selects the highest number possible.
+    check_marginals: bool, optional (default=True)
+        If True, checks that the marginals mass are equal. If False, skips the
+        check.
+
 
     Returns
     -------
@@ -462,15 +481,23 @@ def emd2(a, b, M, processes=1,
     """
 
     a, b, M = list_to_array(a, b, M)
+    nx = get_backend(M, a, b)
 
-    a0, b0, M0 = a, b, M
-    if len(a0) != 0:
-        type_as = a0
-    elif len(b0) != 0:
-        type_as = b0
+    if len(a) != 0:
+        type_as = a
+    elif len(b) != 0:
+        type_as = b
     else:
-        type_as = M0
-    nx = get_backend(M0, a0, b0)
+        type_as = M
+
+    # if empty array given then use uniform distributions
+    if len(a) == 0:
+        a = nx.ones((M.shape[0],), type_as=type_as) / M.shape[0]
+    if len(b) == 0:
+        b = nx.ones((M.shape[1],), type_as=type_as) / M.shape[1]
+
+    # store original tensors
+    a0, b0, M0 = a, b, M
 
     # convert to numpy
     M, a, b = nx.to_numpy(M, a, b)
@@ -479,19 +506,15 @@ def emd2(a, b, M, processes=1,
     b = np.asarray(b, dtype=np.float64)
     M = np.asarray(M, dtype=np.float64, order='C')
 
-    # if empty array given then use uniform distributions
-    if len(a) == 0:
-        a = np.ones((M.shape[0],), dtype=np.float64) / M.shape[0]
-    if len(b) == 0:
-        b = np.ones((M.shape[1],), dtype=np.float64) / M.shape[1]
-
     assert (a.shape[0] == M.shape[0] and b.shape[0] == M.shape[1]), \
         "Dimension mismatch, check dimensions of M with a and b"
 
     # ensure that same mass
-    np.testing.assert_almost_equal(a.sum(0),
-                                   b.sum(0,keepdims=True), err_msg='a and b vector must have the same sum')
-    b = b * a.sum(0) / b.sum(0,keepdims=True)
+    if check_marginals:
+        np.testing.assert_almost_equal(a.sum(0),
+                                       b.sum(0, keepdims=True), err_msg='a and b vector must have the same sum',
+                                       decimal=6)
+    b = b * a.sum(0) / b.sum(0, keepdims=True)
 
     asel = a != 0
 
@@ -631,6 +654,7 @@ def free_support_barycenter(measures_locations, measures_weights, X_init, b=None
 
 
     .. _references-free-support-barycenter:
+
     References
     ----------
     .. [20] Cuturi, Marco, and Arnaud Doucet. "Fast computation of Wasserstein barycenters." International Conference on Machine Learning. 2014.
@@ -688,7 +712,7 @@ def free_support_barycenter(measures_locations, measures_weights, X_init, b=None
 def generalized_free_support_barycenter(X_list, a_list, P_list, n_samples_bary, Y_init=None, b=None, weights=None,
                                         numItermax=100, stopThr=1e-7, verbose=False, log=None, numThreads=1, eps=0):
     r"""
-    Solves the free support generalised Wasserstein barycenter problem: finding a barycenter (a discrete measure with
+    Solves the free support generalized Wasserstein barycenter problem: finding a barycenter (a discrete measure with
     a fixed amount of points of uniform weights) whose respective projections fit the input measures.
     More formally:
 
@@ -776,7 +800,7 @@ def generalized_free_support_barycenter(X_list, a_list, P_list, n_samples_bary,
         Y_init = nx.randn(n_samples_bary, d, type_as=X_list[0])
 
     if b is None:
-        b = nx.ones(n_samples_bary, type_as=X_list[0]) / n_samples_bary  # not optimised
+        b = nx.ones(n_samples_bary, type_as=X_list[0]) / n_samples_bary  # not optimized
 
     out = free_support_barycenter(Z_list, a_list, Y_init, b, numItermax=numItermax,
                                   stopThr=stopThr, verbose=verbose, log=log, numThreads=numThreads)
@@ -786,7 +810,7 @@ def generalized_free_support_barycenter(X_list, a_list, P_list, n_samples_bary,
     else:
         Y = out
         log_dict = None
-    Y = Y @ B.T  # return to the Generalised WB formulation
+    Y = Y @ B.T  # return to the Generalized WB formulation
 
     if log:
         return Y, log_dict
