@@ -37,7 +37,7 @@ def quantile_function(qs, cws, xs):
     n = xs.shape[0]
     if nx.__name__ == 'torch':
         # this is to ensure the best performance for torch searchsorted
-        # and avoid a warninng related to non-contiguous arrays
+        # and avoid a warning related to non-contiguous arrays
         cws = cws.T.contiguous()
         qs = qs.T.contiguous()
     else:
@@ -134,7 +134,7 @@ def wasserstein_1d(u_values, v_values, u_weights=None, v_weights=None, p=1, requ
 
 
 def emd_1d(x_a, x_b, a=None, b=None, metric='sqeuclidean', p=1., dense=True,
-           log=False):
+           log=False, check_marginals=True):
     r"""Solves the Earth Movers distance problem between 1d measures and returns
     the OT matrix
 
@@ -145,6 +145,7 @@ def emd_1d(x_a, x_b, a=None, b=None, metric='sqeuclidean', p=1., dense=True,
         s.t. \gamma 1 = a,
              \gamma^T 1= b,
              \gamma\geq 0
+
     where :
 
     - d is the metric
@@ -180,6 +181,9 @@ def emd_1d(x_a, x_b, a=None, b=None, metric='sqeuclidean', p=1., dense=True,
     log: boolean, optional (default=False)
         If True, returns a dictionary containing the cost.
         Otherwise returns only the optimal transportation matrix.
+    check_marginals: bool, optional (default=True)
+        If True, checks that the marginals mass are equal. If False, skips the
+        check.
 
     Returns
     -------
@@ -219,8 +223,12 @@ def emd_1d(x_a, x_b, a=None, b=None, metric='sqeuclidean', p=1., dense=True,
     ot.lp.emd2_1d : EMD for 1d distributions (returns cost instead of the
         transportation matrix)
     """
-    a, b, x_a, x_b = list_to_array(a, b, x_a, x_b)
+    x_a, x_b = list_to_array(x_a, x_b)
     nx = get_backend(x_a, x_b)
+    if a is not None:
+        a = list_to_array(a, nx=nx)
+    if b is not None:
+        b = list_to_array(b, nx=nx)
 
     assert (x_a.ndim == 1 or x_a.ndim == 2 and x_a.shape[1] == 1), \
         "emd_1d should only be used with monodimensional data"
@@ -234,11 +242,13 @@ def emd_1d(x_a, x_b, a=None, b=None, metric='sqeuclidean', p=1., dense=True,
         b = nx.ones((x_b.shape[0],), type_as=x_b) / x_b.shape[0]
 
     # ensure that same mass
-    np.testing.assert_almost_equal(
-        nx.to_numpy(nx.sum(a, axis=0)),
-        nx.to_numpy(nx.sum(b, axis=0)),
-        err_msg='a and b vector must have the same sum'
-    )
+    if check_marginals:
+        np.testing.assert_almost_equal(
+            nx.to_numpy(nx.sum(a, axis=0)),
+            nx.to_numpy(nx.sum(b, axis=0)),
+            err_msg='a and b vector must have the same sum',
+            decimal=6
+        )
     b = b * nx.sum(a) / nx.sum(b)
 
     x_a_1d = nx.reshape(x_a, (-1,))
@@ -283,6 +293,7 @@ def emd2_1d(x_a, x_b, a=None, b=None, metric='sqeuclidean', p=1., dense=True,
         s.t. \gamma 1 = a,
              \gamma^T 1= b,
              \gamma\geq 0
+
     where :
 
     - d is the metric
@@ -401,7 +412,7 @@ def roll_cols(M, shifts):
 
     n_rows, n_cols = M.shape
 
-    arange1 = nx.tile(nx.reshape(nx.arange(n_cols), (1, n_cols)), (n_rows, 1))
+    arange1 = nx.tile(nx.reshape(nx.arange(n_cols, type_as=shifts), (1, n_cols)), (n_rows, 1))
     arange2 = (arange1 - shifts) % n_cols
 
     return nx.take_along_axis(M, arange2, 1)
@@ -464,7 +475,7 @@ def derivative_cost_on_circle(theta, u_values, v_values, u_cdf, v_cdf, p=2):
 
     if nx.__name__ == 'torch':
         # this is to ensure the best performance for torch searchsorted
-        # and avoid a warninng related to non-contiguous arrays
+        # and avoid a warning related to non-contiguous arrays
         u_cdf = u_cdf.contiguous()
         v_cdf_theta = v_cdf_theta.contiguous()
 
@@ -478,7 +489,7 @@ def derivative_cost_on_circle(theta, u_values, v_values, u_cdf, v_cdf, p=2):
 
     if nx.__name__ == 'torch':
         # this is to ensure the best performance for torch searchsorted
-        # and avoid a warninng related to non-contiguous arrays
+        # and avoid a warning related to non-contiguous arrays
         u_cdfm = u_cdfm.contiguous()
         v_cdf_theta = v_cdf_theta.contiguous()
 
@@ -600,7 +611,7 @@ def binary_search_circle(u_values, v_values, u_weights=None, v_weights=None, p=1
 
     using e.g. ot.utils.get_coordinate_circle(x)
 
-    The function runs on backend but tensorflow is not supported.
+    The function runs on backend but tensorflow and jax are not supported.
 
     Parameters
     ----------
@@ -665,8 +676,8 @@ def binary_search_circle(u_values, v_values, u_weights=None, v_weights=None, p=1
 
     if u_values.shape[1] != v_values.shape[1]:
         raise ValueError(
-            "u and v must have the same number of batchs {} and {} respectively given".format(u_values.shape[1],
-                                                                                              v_values.shape[1]))
+            "u and v must have the same number of batches {} and {} respectively given".format(u_values.shape[1],
+                                                                                               v_values.shape[1]))
 
     u_values = u_values % 1
     v_values = v_values % 1
@@ -730,7 +741,7 @@ def binary_search_circle(u_values, v_values, u_weights=None, v_weights=None, p=1
             tp[((1 - mask) * (dCp >= 0)) > 0] = tc[((1 - mask) * (dCp >= 0)) > 0]
             tc[((1 - mask) * (1 - done)) > 0] = (tm[((1 - mask) * (1 - done)) > 0] + tp[((1 - mask) * (1 - done)) > 0]) / 2
 
-    w = ot_cost_on_circle(tc, u_values, v_values, u_cdf, v_cdf, p)
+    w = ot_cost_on_circle(nx.detach(tc), u_values, v_values, u_cdf, v_cdf, p)
 
     if log:
         return w, {"optimal_theta": tc[:, 0]}
@@ -743,7 +754,7 @@ def wasserstein1_circle(u_values, v_values, u_weights=None, v_weights=None, requ
     takes the value modulo 1.
     If the values are on :math:`S^1\subset\mathbb{R}^2`, first find the coordinates
     using e.g. the atan2 function.
-    The function runs on backend but tensorflow is not supported.
+    The function runs on backend but tensorflow and jax are not supported.
 
     .. math::
         W_1(u,v) = \int_0^1 |F_u(t)-F_v(t)-LevMed(F_u-F_v)|\ \mathrm{d}t
@@ -864,7 +875,7 @@ def wasserstein_circle(u_values, v_values, u_weights=None, v_weights=None, p=1,
 
     using e.g. ot.utils.get_coordinate_circle(x)
 
-    The function runs on backend but tensorflow is not supported.
+    The function runs on backend but tensorflow and jax are not supported.
 
     Parameters
     ----------
