@@ -12,14 +12,14 @@ import numpy as np
 import warnings
 from .lp import emd
 from .bregman import sinkhorn
-from .utils import list_to_array
 from .backend import get_backend
 
 with warnings.catch_warnings():
     warnings.simplefilter("ignore")
     try:
-        from scipy.optimize import scalar_search_armijo
-    except ImportError:
+        from scipy.optimize._linesearch import scalar_search_armijo
+    except ModuleNotFoundError:
+        # scipy<1.8.0
         from scipy.optimize.linesearch import scalar_search_armijo
 
 # The corresponding scipy function does not work for matrices
@@ -27,7 +27,7 @@ with warnings.catch_warnings():
 
 def line_search_armijo(
     f, xk, pk, gfk, old_fval, args=(), c1=1e-4,
-    alpha0=0.99, alpha_min=None, alpha_max=None, nx=None, **kwargs
+    alpha0=0.99, alpha_min=0., alpha_max=None, nx=None, **kwargs
 ):
     r"""
     Armijo linesearch function that works with matrices
@@ -56,7 +56,7 @@ def line_search_armijo(
         :math:`c_1` const in armijo rule (>0)
     alpha0 : float, optional
         initial step (>0)
-    alpha_min : float, optional
+    alpha_min : float, default=0.
         minimum value for alpha
     alpha_max : float, optional
         maximum value for alpha
@@ -73,7 +73,6 @@ def line_search_armijo(
 
     """
     if nx is None:
-        xk, pk, gfk = list_to_array(xk, pk, gfk)
         xk0, pk0 = xk, pk
         nx = get_backend(xk0, pk0)
     else:
@@ -89,11 +88,19 @@ def line_search_armijo(
     fc = [0]
 
     def phi(alpha1):
+        # it's necessary to check boundary condition here for the coefficient
+        # as the callback could be evaluated for negative value of alpha by
+        # `scalar_search_armijo` function here:
+        #
+        # https://github.com/scipy/scipy/blob/11509c4a98edded6c59423ac44ca1b7f28fba1fd/scipy/optimize/linesearch.py#L686
+        #
+        # see more details https://github.com/PythonOT/POT/issues/502
+        alpha1 = np.clip(alpha1, alpha_min, alpha_max)
         # The callable function operates on nx backend
         fc[0] += 1
         alpha10 = nx.from_numpy(alpha1)
         fval = f(xk0 + alpha10 * pk0, *args)
-        if type(fval) is float:
+        if isinstance(fval, float):
             # prevent bug from nx.to_numpy that can look for .cpu or .gpu
             return fval
         else:
@@ -101,7 +108,7 @@ def line_search_armijo(
 
     if old_fval is None:
         phi0 = phi(0.)
-    elif type(old_fval) is float:
+    elif isinstance(old_fval, float):
         # prevent bug from nx.to_numpy that can look for .cpu or .gpu
         phi0 = old_fval
     else:
@@ -109,13 +116,12 @@ def line_search_armijo(
 
     derphi0 = np.sum(pk * gfk)  # Quickfix for matrices
     alpha, phi1 = scalar_search_armijo(
-        phi, phi0, derphi0, c1=c1, alpha0=alpha0)
+        phi, phi0, derphi0, c1=c1, alpha0=alpha0, amin=alpha_min)
 
     if alpha is None:
         return 0., fc[0], nx.from_numpy(phi0, type_as=xk0)
     else:
-        if alpha_min is not None or alpha_max is not None:
-            alpha = np.clip(alpha, alpha_min, alpha_max)
+        alpha = np.clip(alpha, alpha_min, alpha_max)
         return nx.from_numpy(alpha, type_as=xk0), fc[0], nx.from_numpy(phi1, type_as=xk0)
 
 
@@ -138,6 +144,7 @@ def generic_conditional_gradient(a, b, M, f, df, reg1, reg2, lp_solver, line_sea
              \gamma^T \mathbf{1} &= \mathbf{b} (optional constraint)
 
              \gamma &\geq 0
+
     where :
 
     - :math:`\mathbf{M}` is the (`ns`, `nt`) metric cost matrix
@@ -157,6 +164,7 @@ def generic_conditional_gradient(a, b, M, f, df, reg1, reg2, lp_solver, line_sea
              \gamma^T \mathbf{1} &= \mathbf{b}
 
              \gamma &\geq 0
+
     where :
 
     - :math:`\Omega` is the entropic regularization term :math:`\Omega(\gamma)=\sum_{i,j} \gamma_{i,j}\log(\gamma_{i,j})`
@@ -224,10 +232,10 @@ def generic_conditional_gradient(a, b, M, f, df, reg1, reg2, lp_solver, line_sea
 
     See Also
     --------
-    ot.lp.emd : Unregularized optimal ransport
+    ot.lp.emd : Unregularized optimal transport
     ot.bregman.sinkhorn : Entropic regularized optimal transport
     """
-    a, b, M, G0 = list_to_array(a, b, M, G0)
+
     if isinstance(M, int) or isinstance(M, float):
         nx = get_backend(a, b)
     else:
@@ -288,7 +296,7 @@ def generic_conditional_gradient(a, b, M, f, df, reg1, reg2, lp_solver, line_sea
             loop = 0
 
         abs_delta_cost_G = abs(cost_G - old_cost_G)
-        relative_delta_cost_G = abs_delta_cost_G / abs(cost_G)
+        relative_delta_cost_G = abs_delta_cost_G / abs(cost_G) if cost_G != 0. else np.nan
         if relative_delta_cost_G < stopThr or abs_delta_cost_G < stopThr2:
             loop = 0
 
@@ -325,6 +333,7 @@ def cg(a, b, M, reg, f, df, G0=None, line_search=line_search_armijo,
              \gamma^T \mathbf{1} &= \mathbf{b}
 
              \gamma &\geq 0
+
     where :
 
     - :math:`\mathbf{M}` is the (`ns`, `nt`) metric cost matrix
@@ -380,7 +389,7 @@ def cg(a, b, M, reg, f, df, G0=None, line_search=line_search_armijo,
 
     See Also
     --------
-    ot.lp.emd : Unregularized optimal ransport
+    ot.lp.emd : Unregularized optimal transport
     ot.bregman.sinkhorn : Entropic regularized optimal transport
 
     """
@@ -407,6 +416,7 @@ def semirelaxed_cg(a, b, M, reg, f, df, G0=None, line_search=line_search_armijo,
         s.t. \ \gamma \mathbf{1} &= \mathbf{a}
 
              \gamma &\geq 0
+
     where :
 
     - :math:`\mathbf{M}` is the (`ns`, `nt`) metric cost matrix
@@ -492,6 +502,7 @@ def gcg(a, b, M, reg1, reg2, f, df, G0=None, numItermax=10,
              \gamma^T \mathbf{1} &= \mathbf{b}
 
              \gamma &\geq 0
+
     where :
 
     - :math:`\mathbf{M}` is the (`ns`, `nt`) metric cost matrix
