@@ -3,18 +3,16 @@
 # Author: Remi Flamary <remi.flamary@polytechnique.edu>
 #         Nicolas Courty <ncourty@irisa.fr>
 #
+#
 # License: MIT License
 
+import numpy as np
+import pytest
+from numpy.testing import assert_array_almost_equal_nulp
+
 import ot
 import ot.backend
-from ot.backend import torch, jax, cp, tf
-
-import pytest
-
-import numpy as np
-from numpy.testing import assert_array_almost_equal_nulp
-
-from ot.backend import get_backend, get_backend_list, to_numpy
+from ot.backend import get_backend, get_backend_list, jax, tf, to_numpy, torch
 
 
 def test_get_backend_list():
@@ -37,17 +35,7 @@ def test_to_numpy(nx):
     assert isinstance(M2, np.ndarray)
 
 
-def test_get_backend():
-
-    A = np.zeros((3, 2))
-    B = np.zeros((3, 1))
-
-    nx = get_backend(A)
-    assert nx.__name__ == 'numpy'
-
-    nx = get_backend(A, B)
-    assert nx.__name__ == 'numpy'
-
+def test_get_backend_invalid():
     # error if no parameters
     with pytest.raises(ValueError):
         get_backend()
@@ -56,64 +44,38 @@ def test_get_backend():
     with pytest.raises(ValueError):
         get_backend(1, 2.0)
 
-    # test torch
-    if torch:
 
-        A2 = torch.from_numpy(A)
-        B2 = torch.from_numpy(B)
+def test_get_backend(nx):
 
-        nx = get_backend(A2)
-        assert nx.__name__ == 'torch'
+    A = np.zeros((3, 2))
+    B = np.zeros((3, 1))
 
-        nx = get_backend(A2, B2)
-        assert nx.__name__ == 'torch'
+    nx_np = get_backend(A)
+    assert nx_np.__name__ == 'numpy'
 
-        # test not unique types in input
+    A2, B2 = nx.from_numpy(A, B)
+
+    effective_nx = get_backend(A2)
+    assert effective_nx.__name__ == nx.__name__
+
+    effective_nx = get_backend(A2, B2)
+    assert effective_nx.__name__ == nx.__name__
+
+    if nx.__name__ != "numpy":
+        # test that types mathcing different backends in input raise an error
         with pytest.raises(ValueError):
             get_backend(A, B2)
+    else:
+        # Check that subclassing a numpy array does not break get_backend
+        # note: This is only tested for numpy as this is hard to be consistent
+        # with other backends
+        class nx_subclass(nx.__type__):
+            pass
 
-    if jax:
+        A3 = nx_subclass(0)
 
-        A2 = jax.numpy.array(A)
-        B2 = jax.numpy.array(B)
-
-        nx = get_backend(A2)
-        assert nx.__name__ == 'jax'
-
-        nx = get_backend(A2, B2)
-        assert nx.__name__ == 'jax'
-
-        # test not unique types in input
-        with pytest.raises(ValueError):
-            get_backend(A, B2)
-
-    if cp:
-        A2 = cp.asarray(A)
-        B2 = cp.asarray(B)
-
-        nx = get_backend(A2)
-        assert nx.__name__ == 'cupy'
-
-        nx = get_backend(A2, B2)
-        assert nx.__name__ == 'cupy'
-
-        # test not unique types in input
-        with pytest.raises(ValueError):
-            get_backend(A, B2)
-
-    if tf:
-        A2 = tf.convert_to_tensor(A)
-        B2 = tf.convert_to_tensor(B)
-
-        nx = get_backend(A2)
-        assert nx.__name__ == 'tf'
-
-        nx = get_backend(A2, B2)
-        assert nx.__name__ == 'tf'
-
-        # test not unique types in input
-        with pytest.raises(ValueError):
-            get_backend(A, B2)
+        effective_nx = get_backend(A3, B2)
+        assert effective_nx.__name__ == nx.__name__
 
 
 def test_convert_between_backends(nx):
@@ -221,6 +183,8 @@ def test_empty_backend():
         nx.argmin(M)
     with pytest.raises(NotImplementedError):
         nx.mean(M)
+    with pytest.raises(NotImplementedError):
+        nx.median(M)
     with pytest.raises(NotImplementedError):
         nx.std(M)
     with pytest.raises(NotImplementedError):
@@ -296,6 +260,20 @@ def test_empty_backend():
         nx.atan2(v, v)
     with pytest.raises(NotImplementedError):
         nx.transpose(M)
+    with pytest.raises(NotImplementedError):
+        nx.detach(M)
+    with pytest.raises(NotImplementedError):
+        nx.matmul(M, M.T)
+    with pytest.raises(NotImplementedError):
+        nx.nan_to_num(M)
+    with pytest.raises(NotImplementedError):
+        nx.sign(M)
+    with pytest.raises(NotImplementedError):
+        nx.dtype_device(M)
+    with pytest.raises(NotImplementedError):
+        nx.assert_same_dtype_device(M, M)
+    with pytest.raises(NotImplementedError):
+        nx.eigh(M)
 
 
 def test_func_backends(nx):
@@ -306,6 +284,9 @@ def test_func_backends(nx):
     v = rnd.randn(3)
     val = np.array([1.0])
 
+    M1 = rnd.randn(1, 2, 10, 10)
+    M2 = rnd.randn(3, 1, 10, 10)
+
     # Sparse tensors test
     sp_row = np.array([0, 3, 1, 0, 3])
     sp_col = np.array([0, 3, 1, 2, 2])
@@ -324,6 +305,9 @@ def test_func_backends(nx):
         SquareMb = nx.from_numpy(SquareM)
         vb = nx.from_numpy(v)
 
+        M1b = nx.from_numpy(M1)
+        M2b = nx.from_numpy(M2)
+
         val = nx.from_numpy(val)
 
         sp_rowb = nx.from_numpy(sp_row)
@@ -335,6 +319,11 @@ def test_func_backends(nx):
         lst_b.append(nx.to_numpy(A))
         lst_name.append('set_gradients')
 
+        A = nx.detach(Mb)
+        A, B = nx.detach(Mb, Mb)
+        lst_b.append(nx.to_numpy(A))
+        lst_name.append('detach')
+
         A = nx.zeros((10, 3))
         A = nx.zeros((10, 3), type_as=Mb)
         lst_b.append(nx.to_numpy(A))
@@ -395,6 +384,10 @@ def test_func_backends(nx):
         lst_b.append(nx.to_numpy(A))
         lst_name.append('minimum')
 
+        A = nx.sign(vb)
+        lst_b.append(nx.to_numpy(A))
+        lst_name.append('sign')
+
         A = nx.abs(Mb)
         lst_b.append(nx.to_numpy(A))
         lst_name.append('abs')
@@ -431,6 +424,14 @@ def test_func_backends(nx):
         lst_b.append(nx.to_numpy(A))
         lst_name.append('norm')
 
+        A = nx.norm(Mb, axis=1)
+        lst_b.append(nx.to_numpy(A))
+        lst_name.append('norm(M,axis=1)')
+
+        A = nx.norm(Mb, axis=1, keepdims=True)
+        lst_b.append(nx.to_numpy(A))
+        lst_name.append('norm(M,axis=1,keepdims=True)')
+
         A = nx.any(vb > 0)
         lst_b.append(nx.to_numpy(A))
         lst_name.append('any')
@@ -509,11 +510,16 @@ def test_func_backends(nx):
         lst_b.append(nx.to_numpy(A))
         lst_name.append('mean')
 
+        A = nx.median(Mb)
+        lst_b.append(nx.to_numpy(A))
+        lst_name.append('median')
+
         A = nx.std(Mb)
         lst_b.append(nx.to_numpy(A))
         lst_name.append('std')
 
         A = nx.linspace(0, 1, 50)
+        A = nx.linspace(0, 1, 50, type_as=Mb)
         lst_b.append(nx.to_numpy(A))
         lst_name.append('linspace')
 
@@ -533,6 +539,12 @@ def test_func_backends(nx):
         lst_b.append(nx.to_numpy(A))
         lst_name.append('unique')
 
+        A, A2 = nx.unique(nx.from_numpy(np.stack([M, M]).reshape(-1)), return_inverse=True)
+        lst_b.append(nx.to_numpy(A))
+        lst_name.append('unique(M,return_inverse=True)[0]')
+        lst_b.append(nx.to_numpy(A2))
+        lst_name.append('unique(M,return_inverse=True)[1]')
+
         A = nx.logsumexp(Mb)
         lst_b.append(nx.to_numpy(A))
         lst_name.append('logsumexp')
@@ -608,6 +620,10 @@ def test_func_backends(nx):
         lst_b.append(nx.to_numpy(A))
         lst_name.append("matrix square root")
 
+        D, U = nx.eigh(SquareMb.T @ SquareMb)
+        lst_b.append(nx.to_numpy(nx.dot(U, nx.dot(nx.diag(D), U.T))))
+        lst_name.append("eigh ")
+
         A = nx.kl_div(nx.abs(Mb), nx.abs(Mb) + 1)
         lst_b.append(nx.to_numpy(A))
         lst_name.append("Kullback-Leibler divergence")
@@ -649,6 +665,24 @@ def test_func_backends(nx):
         lst_b.append(nx.to_numpy(A))
         lst_name.append("transpose")
 
+        A, B = nx.detach(Mb, Mb)
+        lst_b.append(nx.to_numpy(A))
+        lst_name.append("detach A")
+        lst_b.append(nx.to_numpy(B))
+        lst_name.append("detach B")
+
+        A = nx.matmul(Mb, Mb.T)
+        lst_b.append(nx.to_numpy(A))
+        lst_name.append("matmul")
+        A = nx.matmul(M1b, M2b)
+        lst_b.append(nx.to_numpy(A))
+        lst_name.append("matmul broadcast")
+
+        vec = nx.from_numpy(np.array([1, np.nan, -1]))
+        vec = nx.nan_to_num(vec, nan=0)
+        lst_b.append(nx.to_numpy(vec))
+        lst_name.append("nan_to_num")
+
         assert not nx.array_equal(Mb, vb), "array_equal (shape)"
         assert nx.array_equal(Mb, Mb), "array_equal (elements) - expected true"
         assert not nx.array_equal(
@@ -748,3 +782,11 @@ def test_gradients_backends():
             [dl_dw, dl_db] = tape.gradient(manipulated_loss, [w, b])
             assert nx.allclose(dl_dw, w)
             assert nx.allclose(dl_db, b)
+
+
+def test_get_backend_none():
+    a, b = np.zeros((2, 3)), None
+    nx = get_backend(a, b)
+    assert str(nx) == 'numpy'
+    with pytest.raises(ValueError):
+        get_backend(None, None)
