@@ -74,7 +74,11 @@ def test_emd2_backends(nx):
 
     valb = ot.emd2(ab, ab, Mb)
 
+    # check with empty inputs
+    valb2 = ot.emd2([], [], Mb)
+
     np.allclose(val, nx.to_numpy(valb))
+    np.allclose(val, nx.to_numpy(valb2))
 
 
 def test_emd_emd2_types_devices(nx):
@@ -103,8 +107,6 @@ def test_emd_emd2_types_devices(nx):
 
 @pytest.mark.skipif(not tf, reason="tf not installed")
 def test_emd_emd2_devices_tf():
-    if not tf:
-        return
     nx = ot.backend.TensorflowBackend()
 
     n_samples = 100
@@ -299,7 +301,7 @@ def test_lp_barycenter():
     A = np.hstack((a1, a2))
     M = np.array([[0, 1.0, 4.0], [1.0, 0, 1.0], [4.0, 1.0, 0]])
 
-    # obvious barycenter between two diracs
+    # obvious barycenter between two Diracs
     bary0 = np.array([0, 1.0, 0])
 
     bary = ot.lp.barycenter(A, M, [.5, .5])
@@ -314,7 +316,7 @@ def test_free_support_barycenter():
 
     X_init = np.array([-12.]).reshape((1, 1))
 
-    # obvious barycenter location between two diracs
+    # obvious barycenter location between two Diracs
     bar_locations = np.array([0.]).reshape((1, 1))
 
     X = ot.lp.free_support_barycenter(measures_locations, measures_weights, X_init)
@@ -340,7 +342,6 @@ def test_free_support_barycenter_backends(nx):
 
 
 def test_generalised_free_support_barycenter():
-    np.random.seed(42)  # random inits
     X = [np.array([-1., -1.]).reshape((1, 2)), np.array([1., 1.]).reshape((1, 2))]  # two 2D points bar is obviously 0
     a = [np.array([1.]), np.array([1.])]
 
@@ -348,7 +349,7 @@ def test_generalised_free_support_barycenter():
 
     Y_init = np.array([-12., 7.]).reshape((1, 2))
 
-    # obvious barycenter location between two 2D diracs
+    # obvious barycenter location between two 2D Diracs
     Y_true = np.array([0., .0]).reshape((1, 2))
 
     # test without log and no init
@@ -361,7 +362,6 @@ def test_generalised_free_support_barycenter():
 
 
 def test_generalised_free_support_barycenter_backends(nx):
-    np.random.seed(42)
     X = [np.array([-1.]).reshape((1, 1)), np.array([1.]).reshape((1, 1))]
     a = [np.array([1.]), np.array([1.])]
     P = [np.array([1.]).reshape((1, 1)), np.array([1.]).reshape((1, 1))]
@@ -387,7 +387,7 @@ def test_lp_barycenter_cvxopt():
     A = np.hstack((a1, a2))
     M = np.array([[0, 1.0, 4.0], [1.0, 0, 1.0], [4.0, 1.0, 0]])
 
-    # obvious barycenter between two diracs
+    # obvious barycenter between two Diracs
     bary0 = np.array([0, 1.0, 0])
 
     bary = ot.lp.barycenter(A, M, [.5, .5], solver=None)
